package induction_examples;

import java.util.ArrayList;
import java.util.List;

/**
 * BankAccount represents a bank account.
 * 
 * @specfield balance: double                 // Current balance in account
 * @specfield transactions: List<Transaction> // All transactions affecting the
 *                                            // balance in this account
 * 
 * Abstract invariant: balance >= 0
 */
public class BankAccount {
    public List<Transaction> transactions;
    
    // Abstraction function:
    // A BankAccount r represents a bank account b where
    //      b.balance = sum(t.amount) for all t in r.transactions
    //      b.transactions = this.transactions

    // Rep invariant:
    // transactions contains no null values
    // the sum of all values in transactions is >= 0


    /**
     * @effects constructs a new BankAccount with no transactions and a balance of 0
     */
    public BankAccount() {
        transactions = new ArrayList<Transaction>();
        checkRep();
    }

    /**
     * @return balance of account
     */
    public double getBalance() {
        double total = 0;
        for (Transaction t : transactions)
            total += t.getAmount();
        return total;
    }

    /**
     * @param txn
     *            The transaction to be performed on this BankAccount
     * @modifies balance, transactions
     * @effects if (balance + txn.amount) >= 0, then balance = balance + txn.amount
     *              and transactions = transaction + txn;
     *          else, balance and transactions are left unchanged
     * @throws IllegalArgumentException
     *             if the transaction would cause the balance to become negative
     * @throws NullPointerException
     *             if txn is null
     */
    public void performTxn(Transaction txn) {
        if (txn == null)
          throw new NullPointerException("txn cannot be null");
        if (getBalance() + txn.getAmount() < 0)
            throw new IllegalArgumentException("Balance of account cannot be negative");
        transactions.add(txn);
    }

    private void checkRep() {
        assert transactions != null;
        assert !transactions.contains(null);
        double total = 0;
        for (Transaction t : transactions)
            total += t.getAmount();
        assert total >= 0;
    }
}