package induction_examples;

/**
 * TreeSet represents a mutable set of values of type K.
 * 
 */
public class TreeSet<K extends Comparable<K>> {
    private TreeSet<K> left, right;
    private K data;

    // Abstraction function:
    // AF(r) = a set S, such that
    // If data is null, S = {} (empty set)
    // Else, S = {data} U AF(left) U AF(right)

    // Representation invariant:
    // data == null iff (left == null and right == null)
    // if data != null, all non-null values in tree rooted at left are < data
    // and all non-null values in tree rooted at right are > data

    /**
     * @effects Constructs an empty TreeSet.
     */
    public TreeSet() {
        left = null;
        right = null;
        data = null;
    }

    /**
     * Checks whether a value is in the set.
     * 
     * @param val
     *            The value to search for.
     * @return true iff val is contained in values.
     * @throws NullPointerException
     *             if val is null.
     */
    public boolean contains(K val) {
        if (val == null)
            throw new NullPointerException("Cannot look up null value");
        if (data == null)
            return false;
        else if (val.equals(data))
            return true;
        else if (val.compareTo(data) < 0)
            return left.contains(val);
        else
            return right.contains(val);
    }

    /**
     * Add a value to the set if it is not present already.
     * 
     * @modifies this
     * @effects if val is not in the set, then this = this U val; otherwise,
     *          no effect.
     * @param val
     *            Value to be added
     * @throws NullPointerException
     *             if val is null
     */
    public void add(K val) {
        if (val == null)
            throw new NullPointerException("Cannot add null value");
        if (data == null) {
            data = val;
            left = new TreeSet<K>();
            right = new TreeSet<K>();
        } else if (val.compareTo(data) < 0) {
            left.add(val);
        } else if (val.compareTo(data) > 0) {
            right.add(val);
        }
    }
}
